@@ -8,7 +8,7 @@ def handler(signum, frame):
 # Set the signal handler
 signal.signal(signal.SIGINT, handler)
 
-from pykin.robots.gripper import Gripper
+from pykin.robots.gripper import PandaGripper, Robotiq140Gripper
 from pykin.kinematics.transform import Transform
 from pykin.kinematics.kinematics import Kinematics
 from pykin.models.urdf_model import URDFModel
@@ -26,7 +26,8 @@ class Robot(URDFModel):
         self,
         f_name,
         offset, 
-        has_gripper
+        has_gripper,
+        gripper_name
     ):
         super(Robot, self).__init__(f_name)
 
@@ -34,6 +35,7 @@ class Robot(URDFModel):
         if offset is None:
             self._offset = Transform()
         self.has_gripper = has_gripper
+        self.gripper_name = gripper_name
             
         self.urdf_name = os.path.abspath(self.file_path)
         self.mesh_path = os.path.abspath(self.file_path + "/../") + '/'
@@ -41,7 +43,10 @@ class Robot(URDFModel):
         self.gripper = None
 
         if has_gripper:
-            self.gripper = Gripper()
+            if "panda" in self.gripper_name:
+                self.gripper = PandaGripper()
+            if "robotiq140" in self.gripper_name:
+                self.gripper = Robotiq140Gripper()
 
         self.joint_limits_lower = []
         self.joint_limits_upper = []
@@ -70,7 +75,7 @@ class Robot(URDFModel):
             self.info["visual"][link][3] = visual_h_mat
             
             if self.has_gripper:
-                if link in self.gripper.names:
+                if link in self.gripper.element_names:
                     self.gripper.info[link][3] = collision_h_mat
 
     def show_robot_info(self):
@@ -139,14 +144,24 @@ class Robot(URDFModel):
     def _init_gripper_info(self):
         gripper_info = {}
         for link, transform in self.init_fk.items():
-            if link in self.gripper.names:
+            col_gparam = []
+            if link in self.gripper.element_names:
                 gtype = self.links[link].collision.gtype
                 mesh = None
                 if gtype == "mesh":
                     mesh_path = self.mesh_path + self.links[link].collision.gparam.get('filename')
                     mesh = trimesh.load_mesh(mesh_path)
+                    col_gparam.append(mesh)
+                if gtype == "box":
+                    col_gparam.append(self.links[link].collision.gparam.get('size'))
+                if gtype == "cylinder":
+                    length = float(self.links[link].collision.gparam.get('length'))
+                    radius = float(self.links[link].collision.gparam.get('radius'))
+                    col_gparam.append((length, radius))
+                if gtype == "sphere":
+                    col_gparam.append(float(self.links[link].collision.gparam.get('radius')))
                 h_mat = np.dot(transform.h_mat, self.links[link].collision.offset.h_mat)
-                gripper_info[link] = [link, gtype, mesh, h_mat]
+                gripper_info[link] = [link, gtype, col_gparam, h_mat]
         return gripper_info
 
     def _setup_kinematics(self):
@@ -240,6 +255,25 @@ class Robot(URDFModel):
         """
         return compute_pose_error(target, result)
 
+    def open_gripper(self, z_dis=0.02):
+        for geom in ['collision', 'visual']:
+            for finger in self.gripper.finger_names:
+                if "robotiq140" in self.gripper_name:
+                    self.info[geom][finger][3][:3, 3] = self.info[geom][finger][3][:3, 3] + z_dis * self.info[geom][finger][3][:3,2]
+                if "panda" in self.gripper_name:
+                    self.info[geom][finger][3][:3, 3] = self.info[geom][finger][3][:3, 3] + z_dis * self.info[geom][finger][3][:3,1]
+        
+        self.gripper.open_gripper(z_dis)
+
+    def close_gripper(self, z_dis=0.02):        
+        for geom in ['collision', 'visual']:
+            for finger in self.gripper.finger_names:
+                if "robotiq140" in self.gripper_name:
+                    self.info[geom][finger][3][:3, 3] = self.info[geom][finger][3][:3, 3] - z_dis * self.info[geom][finger][3][:3,2]
+                if "panda" in self.gripper_name:
+                    self.info[geom][finger][3][:3, 3] = self.info[geom][finger][3][:3, 3] - z_dis * self.info[geom][finger][3][:3,1]
+        self.gripper.close_gripper(z_dis)
+            
     @property
     def offset(self):
         return self._offset
