@@ -1,10 +1,42 @@
 import numpy as np
+import os, datetime
 import matplotlib.pyplot as plt
 
 from mpl_toolkits.mplot3d.art3d import Poly3DCollection
 from pykin.utils import transform_utils as t_utils
 
 
+def init_2d_figure(name=None, figsize=(15,7.5), dpi= 80):
+    fig = plt.figure(name, figsize=figsize, dpi= dpi)
+    ax = plt.gca()
+    return fig, ax
+
+
+def createDirectory(directory): 
+    try: 
+        if not os.path.exists(directory): 
+            print(f"Create {directory} direcoty")
+            os.makedirs(directory) 
+    except OSError: print("Error: Failed to create the directory.")
+
+
+def plot_values(ax, values, label="", title="result", is_save=False, save_dir_name="result_images", **configs):
+    if configs:
+        plt.title(f'Benchmark: {configs["num"]} (Algo: {configs["algo"]}, Constant: {configs["c"]})', loc='center')
+    ax.plot(values, label=label)
+    plt.xticks(fontsize=10)
+    plt.yticks(fontsize=10)
+    plt.xlabel("Number of simulations",fontsize=12)
+    plt.ylabel("Max Value",fontsize=12)
+    ax.legend(prop={'size' : 12})
+
+    if is_save:
+        createDirectory(save_dir_name)
+        file_name = save_dir_name + '/' + title + '_{}.png'.format(datetime.datetime.now().strftime("%Y%m%d_%H%M%S"))    
+        print(f"Save {file_name}")
+        plt.savefig(file_name)
+
+
 def init_3d_figure(name=None, figsize=(12,8), dpi=100, visible_axis=False):
     """
     Initializes 3d figure
@@ -26,7 +58,6 @@ def show_figure():
     plt.show()
 
 
-
 def plot_basis(ax=None, robot=None):
     """
     Plot a frame fitted to the robot size
@@ -153,6 +184,7 @@ def plot_robot(
             ax.scatter([x[0] for x in nodes], [x[1] for x in nodes],
                 [x[2] for x in nodes], s=20, c=lines[0].get_color())
 
+
 def plot_attached_object(ax, robot, alpha):
     plot_mesh(
         ax, 
@@ -161,6 +193,7 @@ def plot_attached_object(ax, robot, alpha):
         alpha=alpha,
         color=robot.gripper.info[robot.gripper.attached_obj_name][4])
 
+
 def plot_geom(ax, robot, geom="collision", alpha=0.4, color=None):
     """
     Plot robot's collision
@@ -168,34 +201,38 @@ def plot_geom(ax, robot, geom="collision", alpha=0.4, color=None):
 
     plot_basis(ax, robot)
     for link, info in robot.info[geom].items():
-        h_mat = info[3]
-        if info[1] == 'mesh':
-            meshes = np.array([info[2]]).reshape(-1)
-            for mesh in meshes:
-                mesh_color = get_mesh_color(robot, link, geom, color)
-                plot_mesh(ax, mesh=mesh, h_mat=h_mat, alpha=alpha, color=mesh_color)
+        plot_geom_from_info(ax, robot, link, geom, info, alpha, color)
 
-        if info[1] == 'cylinder':
-            for param in info[2]:
-                length = float(param[0])
-                radius = float(param[1])
-                cylinder_color = get_color(robot.links[link].visual.gparam)
-                plot_cylinder(ax, length=length, radius=radius, h_mat=h_mat, alpha=alpha, color=cylinder_color)
 
-        if info[1] == 'sphere':
-            for param in info[2]:
-                length = float(param)
-                radius = float(param)
-                pos = h_mat[:3,-1]
-                sphere_color = get_color(robot.links[link].visual.gparam)
-                plot_sphere(ax, radius=radius, center_point=pos, n_steps=20, alpha=alpha, color=sphere_color)
-        
-        if info[1] == 'box':
-            for param in info[2]:
-                size = param
-                box_color = get_color(robot.links[link].visual.gparam)
-                plot_box(ax, size, h_mat=h_mat, alpha=alpha, color=box_color)
+def plot_geom_from_info(ax, robot, link, geom, info, alpha, color):
+    h_mat = info[3]
+    if info[1] == 'mesh':
+        meshes = np.array([info[2]]).reshape(-1)
+        for idx, mesh in enumerate(meshes):
+            mesh_color = get_mesh_color(robot, link, geom, idx, color)
+            plot_mesh(ax, mesh=mesh, h_mat=h_mat, alpha=alpha, color=mesh_color)
+
+    if info[1] == 'cylinder':
+        for idx, param in enumerate(info[2]):
+            length = float(param[0])
+            radius = float(param[1])
+            cylinder_color = get_color(robot.links[link].visual.gparam, idx)
+            plot_cylinder(ax, length=length, radius=radius, h_mat=h_mat, alpha=alpha, color=cylinder_color)
+
+    if info[1] == 'sphere':
+        for idx, param in enumerate(info[2]):
+            length = float(param)
+            radius = float(param)
+            pos = h_mat[:3,-1]
+            sphere_color = get_color(robot.links[link].visual.gparam, idx)
+            plot_sphere(ax, radius=radius, center_point=pos, n_steps=20, alpha=alpha, color=sphere_color)
     
+    if info[1] == 'box':
+        for idx, param in enumerate(info[2]):
+            size = param
+            box_color = get_color(robot.links[link].visual.gparam, idx)
+            plot_box(ax, size, h_mat=h_mat, alpha=alpha, color=box_color)
+
 
 def plot_objects(ax, objects, alpha=0.5):    
     """
@@ -206,10 +243,12 @@ def plot_objects(ax, objects, alpha=0.5):
         o_param = info.gparam
         o_pose = info.h_mat
         if o_type == "mesh":
-            if info.name in ["table", "tray", "shelf_9"]:
-                _alpha = 0.2
-            else:
-                _alpha = alpha
+            for obj_name in ["table", "tray", "shelf_8", "shelf_9", "shelf_15", "clearbox"]:
+                if obj_name in info.name:
+                    _alpha = 0.2
+                    break
+                else:
+                    _alpha = alpha
             plot_mesh(ax, mesh=o_param, h_mat=o_pose, alpha=_alpha, color=info.color)
         if o_type == "sphere":
             plot_sphere(ax, radius=o_param, center_point=o_pose, alpha=alpha, color=info.color)
@@ -221,7 +260,7 @@ def plot_objects(ax, objects, alpha=0.5):
             plot_cylinder(ax, radius=o_param[0], length=o_param[1], h_mat=h_mat, n_steps=100, alpha=alpha, color=info.color)
 
 
-def plot_object(ax, obj, pose=None, alpha=0.5, color='k'):    
+def plot_object(ax, obj, pose=None, alpha=0.5):    
     """
     Plot objects
     """
@@ -257,29 +296,33 @@ def render_axis(
             plot_normal_vector(ax, pose[:3, 3], pose[:3, 2], scale=scale, edgecolor="blue")
 
 
-def get_mesh_color(robot, link, geom, color=None):
-    mesh_color = None
+def get_mesh_color(robot, link, geom, idx=0, color=None):
+    mesh_color = color
     if color is None:
+        mesh_color = np.array([0.2, 0.2, 0.2, 1])
+        robot_link = robot.links.get(link)
         if geom == "collision":
-            link = robot.links.get(link)
-            if link is not None:
-                mesh_color = link.collision.gparam.get('color')            
-            if mesh_color is None:
-                mesh_color = np.array([0.2, 0.2, 0.2, 1])
-            else:
-                mesh_color = np.array([color for color in mesh_color.values()]).flatten()
+            if robot_link:
+                if robot_link.collision.gparam.get('color'):
+                    mesh_color = robot_link.collision.gparam.get('color')[idx]
+                    mesh_color = np.array([color for color in mesh_color.values()]).flatten()
+            else: 
+                if robot.has_gripper:
+                    if robot.gripper.info.get(robot.gripper.attached_obj_name) is not None:
+                        mesh_color = robot.gripper.info.get(robot.gripper.attached_obj_name)[4]
         else:
-            link = robot.links.get(link)
-            if link is not None:
-                mesh_color = link.visual.gparam.get('color')
-            if mesh_color is None:
-                mesh_color = np.array([0.2, 0.2, 0.2, 1])
-            else:
-                mesh_color = np.array([color for color in mesh_color.values()]).flatten()
+            if robot_link:
+                if robot_link.visual.gparam.get('color'):
+                    mesh_color = robot_link.visual.gparam.get('color')[idx]
+                    mesh_color = np.array([color for color in mesh_color.values()]).flatten()
+                else: 
+                    if robot.has_gripper:
+                        if robot.gripper.info.get(robot.gripper.attached_obj_name) is not None:
+                            mesh_color = robot.gripper.info.get(robot.gripper.attached_obj_name)[4]
     return mesh_color
 
 
-def get_color(params):
+def get_color(params, idx=0):
     def convert_color_type(color):
         if isinstance(color, str):
             color = color
@@ -304,10 +347,10 @@ def get_color(params):
         return color
         
     color = []
-    if params is not None:
-        visual_color = params.get('color')
+    if params is not None and params.get('color'):
+        visual_color = params.get('color')[idx]
         if visual_color is not None:
-            color = list(visual_color.keys())
+            color = list(visual_color.values())
     color = convert_color_type(color)
     return color
     
@@ -497,4 +540,4 @@ def plot_trajectories(ax, path, size=3, color='r'):
     """
     Plot plot_trajectories
     """
-    ax.scatter([x for (x, y, z) in path], [y for (x, y, z) in path], [z for (x, y, z) in path], s=size, c=color)
+    ax.scatter([x for (x, y, z) in path], [y for (x, y, z) in path], [z for (x, y, z) in path], s=size, c=color)
\ No newline at end of file
